// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.data

import co.vulcanlabs.psremote.common.MacAddress
import co.vulcanlabs.psremote.common.ManualHost
import co.vulcanlabs.psremote.common.RegisteredHost
import co.vulcanlabs.psremote.data.local.AppDatabase
import io.reactivex.rxjava3.core.Flowable
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import javax.inject.Inject

class DeviceRepositoryImpl @Inject constructor(private val appDatabase: AppDatabase) :
    DeviceRepository {
    override suspend fun insertRegisterHost(registeredHost: RegisteredHost): Result<Unit> =
        withContext(Dispatchers.IO) {
            try {
                appDatabase.registeredHostDao().insert(registeredHost)
                Result.Success(Unit)
            } catch (e: Exception) {
                Result.Failure(e)
            }
        }

    override suspend fun getRegisterHostByMac(mac: MacAddress): Result<RegisteredHost> =
        withContext(Dispatchers.IO) {
            try {
                val registeredHost = appDatabase.registeredHostDao().getByMac(mac)
                if (registeredHost != null) {
                    Result.Success(registeredHost)
                } else {
                    Result.Failure(NotFoundException)
                }
            } catch (e: Exception) {
                Result.Failure(e)
            }
        }

    override suspend fun insertManualHost(manualHost: ManualHost): Result<Unit> =
        withContext(Dispatchers.IO) {
            try {
                appDatabase.manualHostDao().insert(manualHost)
                Result.Success(Unit)
            } catch (e: Exception) {
                Result.Failure(e)
            }
        }

    override fun getAllRegisteredHost(): Flowable<List<RegisteredHost>> =
        appDatabase.registeredHostDao().getAll()

    override fun getAllManualHost(): Flowable<List<ManualHost>> =
        appDatabase.manualHostDao().getAll()

    override suspend fun deleteManualHost(manualHost: ManualHost): Result<Unit> =
        withContext(Dispatchers.IO) {
            try {
                appDatabase.manualHostDao().delete(manualHost)
                Result.Success(Unit)
            } catch (e: Exception) {
                Result.Failure(e)
            }
        }

    override suspend fun clearAllDevices(): Result<Unit> = withContext(Dispatchers.IO) {
        try {
            appDatabase.registeredHostDao().deleteAll()
            appDatabase.manualHostDao().deleteAll()
            Result.Success(Unit)
        } catch (e: Exception) {
            Result.Failure(e)
        }
    }

    override fun getRegisteredHostByMacSync(macAddress: MacAddress): Result<RegisteredHost> {
        return appDatabase.registeredHostDao().getByMacSync(macAddress)?.let {
            Result.Success(it)
        } ?: Result.Failure(NotFoundException)
    }
}