// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.data.local

import android.app.Application
import androidx.annotation.StringRes
import co.vulcanlabs.psremote.R
import co.vulcanlabs.psremote.chiaki.ConnectVideoProfile
import co.vulcanlabs.psremote.chiaki.VideoFPSPreset
import co.vulcanlabs.psremote.chiaki.VideoResolutionPreset
import co.vulcanlabs.psremote.common.UserAccount
import co.vulcanlabs.psremote.data.BaseSharePreference
import co.vulcanlabs.psremote.utils.jsonToObj
import co.vulcanlabs.psremote.utils.toJson
import javax.inject.Inject
import javax.inject.Singleton
import kotlin.math.max
import kotlin.math.min

@Singleton
class Preferences @Inject constructor(private val app: Application) : BaseSharePreference(app) {

    companion object {
        val resolutionDefault = Resolution.RES_720P
        val resolutionAll = Resolution.values()
        val fpsDefault = FPS.FPS_60
        val fpsAll = FPS.values()
        val codecDefault = Codec.CODEC_H265
        val codecAll = Codec.values()
    }

    enum class Resolution(
        val value: String,
        @StringRes val title: Int,
        val preset: VideoResolutionPreset
    ) {
        RES_360P(
            "360p",
            R.string.preferences_resolution_title_360p,
            VideoResolutionPreset.RES_360P
        ),
        RES_540P(
            "540p",
            R.string.preferences_resolution_title_540p,
            VideoResolutionPreset.RES_540P
        ),
        RES_720P(
            "720p",
            R.string.preferences_resolution_title_720p,
            VideoResolutionPreset.RES_720P
        ),
        RES_1080P(
            "1080p",
            R.string.preferences_resolution_title_1080p,
            VideoResolutionPreset.RES_1080P
        ),
    }

    enum class FPS(val value: String, @StringRes val title: Int, val preset: VideoFPSPreset) {
        FPS_30("30", R.string.preferences_fps_title_30, VideoFPSPreset.FPS_30),
        FPS_60("60", R.string.preferences_fps_title_60, VideoFPSPreset.FPS_60)
    }

    enum class Codec(
        val value: String,
        @StringRes val title: Int,
        val codec: co.vulcanlabs.psremote.chiaki.Codec
    ) {
        CODEC_H264(
            "h264",
            R.string.preferences_codec_title_h264,
            co.vulcanlabs.psremote.chiaki.Codec.CODEC_H264
        ),
        CODEC_H265(
            "h265",
            R.string.preferences_codec_title_h265,
            co.vulcanlabs.psremote.chiaki.Codec.CODEC_H265
        )
    }

    private val codecKey get() = app.resources.getString(R.string.preferences_codec_key)
    var codec
        get() = getData(codecKey, codecDefault.value).let { value ->
            Codec.values().firstOrNull { it.value == value }
        } ?: codecDefault
        set(value) {
            storeData(codecKey, value.value)
        }

    private val resolutionKey get() = app.resources.getString(R.string.preferences_resolution_key)
    var resolution
        get() = getData(resolutionKey, resolutionDefault.value).let { value ->
            Resolution.values().firstOrNull { it.value == value }
        } ?: resolutionDefault
        set(value) {
            storeData(resolutionKey, value.value)
        }

    private val fpsKey get() = app.resources.getString(R.string.preferences_fps_key)
    var fps
        get() = getData(fpsKey, fpsDefault.value).let { value ->
            FPS.values().firstOrNull { it.value == value }
        } ?: fpsDefault
        set(value) {
            storeData(fpsKey, value.value)
        }

    private val videoProfileDefaultBitrate
        get() = ConnectVideoProfile.preset(
            resolution.preset,
            fps.preset,
            codec.codec
        )
    val videoProfile
        get() = videoProfileDefaultBitrate.let {
            val bitrate = bitrate
            if (bitrate == null)
                it
            else
                it.copy(bitrate = bitrate)
        }

    private fun validateBitrate(bitrate: Int) = max(2000, min(50000, bitrate))
    private val bitrateKey get() = app.resources.getString(R.string.preferences_bitrate_key)
    var bitrate
        get() = getData(bitrateKey, 0).let { if (it == 0) null else validateBitrate(it) }
        set(value) {
            storeData(bitrateKey, if (value != null) validateBitrate(value) else 0)
        }

    // Play setting
    private val playSettingOrientationLockModeKey get() = "PLAY_SETTING_ORIENTATION_LOCK_MODE"
    var playSettingOrientationLockMode
        get() = getData(playSettingOrientationLockModeKey, "AUTO_ROTATE")
        set(value) {
            storeData(playSettingOrientationLockModeKey, value)
        }

    private val playSettingAlwaysShowControllerKey get() = "PLAY_SETTING_ALWAYS_SHOW_CONTROLLER"
    var playSettingAlwaysShowController
        get() = getData(playSettingAlwaysShowControllerKey, true)
        set(value) {
            storeData(playSettingAlwaysShowControllerKey, value)
        }

    private val currentUserKey get() = "CURRENT_USER"
    var currentUser: UserAccount?
        get() = try {
            getData(currentUserKey, "").jsonToObj<UserAccount>()
        } catch (e: Exception) {
            null
        }
        set(value) {
            storeData(currentUserKey, value?.toJson().orEmpty())
        }
}