// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.data.network

import android.app.Application
import android.util.Log
import co.vulcanlabs.psremote.utils.getGsonInstance
import co.vulcanlabs.psremote.utils.isNetworkConnected
import com.readystatesoftware.chuck.ChuckInterceptor
import hu.akarnokd.rxjava3.retrofit.RxJava3CallAdapterFactory
import okhttp3.*
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import java.util.concurrent.TimeUnit

open class BaseNetwork(
    val app: Application,
    val url: String,
    val timeout: Int,
    val useCache: Boolean,
    val showDebugNotification: Boolean,
    val interceptors: List<(Interceptor.Chain) -> Response> = listOf(),
    val cacheSize: Long = (5 * 1024 * 1024).toLong()
) {
    inline fun <reified T> getService(): T {
        val myCache = Cache(app.cacheDir, cacheSize)

        val okHttpClientBuilder =
            OkHttpClient.Builder()
                .connectTimeout(timeout.toLong(), TimeUnit.SECONDS)
                .readTimeout(timeout.toLong(), TimeUnit.SECONDS)
                .writeTimeout(timeout.toLong(), TimeUnit.SECONDS)

        if (useCache) {
            okHttpClientBuilder.cache(myCache)
                .addInterceptor { chain ->
                    // Get the request from the chain.
                    var request = chain.request()
                    val control = CacheControl.Builder()
                    if (app.isNetworkConnected()) {
                        val cacheTime = request.header("AppCached")?.trim()
                        if (!cacheTime.isNullOrEmpty()) {
                            control.maxStale(cacheTime.toIntOrNull() ?: 5, TimeUnit.SECONDS)
                        }
                    }

                    request = request.newBuilder()
                        .cacheControl(control.build())
                        .removeHeader("AppCached")
                        .build()

                    chain.proceed(request)
                }
        }

        interceptors.forEach { interceptor ->
            okHttpClientBuilder.addInterceptor {
                interceptor.invoke(
                    it
                )
            }
        }

        okHttpClientBuilder//logging
            .addInterceptor(
                HttpLoggingInterceptor(object : HttpLoggingInterceptor.Logger {
                    override fun log(message: String) {
                        Log.d(T::class.java.simpleName, message)
                    }
                })
                    .setLevel(HttpLoggingInterceptor.Level.BODY)
            )

        if (showDebugNotification) {
            okHttpClientBuilder.addInterceptor(ChuckInterceptor(app))
        }

        val retrofit = Retrofit.Builder()
            .client(okHttpClientBuilder.build())
            .baseUrl(url)
            .addCallAdapterFactory(RxJava3CallAdapterFactory.create())
            .addConverterFactory(GsonConverterFactory.create(getGsonInstance()))
            .build()

        return retrofit.create(T::class.java)
    }
}