// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.management

import android.util.Log
import co.vulcanlabs.psremote.chiaki.CreateError
import co.vulcanlabs.psremote.chiaki.DiscoveryHost
import co.vulcanlabs.psremote.chiaki.DiscoveryService
import co.vulcanlabs.psremote.chiaki.DiscoveryServiceOptions
import co.vulcanlabs.psremote.common.DiscoveredDisplayHost
import co.vulcanlabs.psremote.common.MacAddress
import co.vulcanlabs.psremote.data.DeviceRepository
import co.vulcanlabs.psremote.data.Result
import co.vulcanlabs.psremote.utils.hexToByteArray
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.core.Observable
import io.reactivex.rxjava3.disposables.CompositeDisposable
import io.reactivex.rxjava3.kotlin.addTo
import io.reactivex.rxjava3.schedulers.Schedulers
import io.reactivex.rxjava3.subjects.BehaviorSubject
import io.reactivex.rxjava3.subjects.Subject
import java.net.InetSocketAddress
import java.nio.charset.StandardCharsets
import java.util.*
import java.util.concurrent.TimeUnit
import javax.inject.Inject
import javax.inject.Singleton

val DiscoveryHost.serverMac
    get() = this.hostId?.hexToByteArray()?.let {
        if (it.size == MacAddress.LENGTH)
            MacAddress(it)
        else
            null
    }

@Singleton
class DiscoveryManager @Inject constructor(private val deviceRepository: DeviceRepository) {
    companion object {
        const val HOSTS_MAX: ULong = 16U
        const val DROP_PINGS: ULong = 3U
        const val PING_MS: ULong = 500U
        const val PORT = 987

        const val DEBOUNCE_EMPTY_MS = 1000L
    }

    var onGameChange: (() -> Unit)? = null

    private var discoveryService: DiscoveryService? = null

    private val discoveryActiveSubject: Subject<Boolean> =
        BehaviorSubject.create<Boolean>().also { it.onNext(true) }
    val discoveryActive: Observable<Boolean> get() = discoveryActiveSubject
    var active = true
        set(value) {
            field = value
            discoveryActiveSubject.onNext(value)
            updateService()
        }
    private var paused = false

    private val disposable = CompositeDisposable()

    private val discoveredHostsSubjectDebounced: Subject<List<DiscoveryHost>> =
        BehaviorSubject.create<List<DiscoveryHost>>().also {
            it.onNext(listOf())
        }.toSerialized()

    private val prevDiscoveredHost = Collections.synchronizedList(mutableListOf<DiscoveryHost>())

    private val discoveredHostsSubjectRaw: Subject<List<DiscoveryHost>> =
        BehaviorSubject.create<List<DiscoveryHost>>().also { subject ->
            subject.debounce { hosts ->
                if (hosts.isEmpty())
                    Observable.timer(DEBOUNCE_EMPTY_MS, TimeUnit.MILLISECONDS)
                else
                    Observable.empty()
            }
                .observeOn(Schedulers.io())
                .doOnNext {
                    prevDiscoveredHost.clear()
                    prevDiscoveredHost.addAll(it)
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe { hosts ->
                    val currentDevice = DeviceManager.getCurrentDevice()
                    if (currentDevice is DiscoveredDisplayHost) {
                        // Update app name and app id
                        hosts.find { it.hostId != null && currentDevice.discoveredHost.hostId == it.hostId }
                            ?.let {
                                if (currentDevice.discoveredHost.runningAppTitleid != it.runningAppTitleid) {
                                    onGameChange?.invoke()
                                }
                                currentDevice.discoveredHost.runningAppTitleid =
                                    it.runningAppTitleid
                                currentDevice.discoveredHost.runningAppName = it.runningAppName
                            }
                    }
                    discoveredHostsSubjectDebounced.onNext(hosts)
                }
                .addTo(disposable)
        }

    val discoveredHosts: Observable<List<DiscoveryHost>> get() = discoveredHostsSubjectDebounced

    init {
        active = true
    }

    fun resume() {
        paused = false
        updateService()
    }

    fun pause() {
        paused = true
        updateService()
    }

    fun dispose() {
        active = false
        disposable.dispose()
    }

    fun refresh() {
        discoveryService = null
        paused = false
        active = true
    }


    fun sendWakeup(host: String, registKey: ByteArray, ps5: Boolean) {
        val registKeyString = registKey.indexOfFirst { it == 0.toByte() }
            .let { end -> registKey.copyOfRange(0, if (end >= 0) end else registKey.size) }
            .toString(
                StandardCharsets.UTF_8
            )
        val credential = try {
            registKeyString.toULong(16)
        } catch (e: NumberFormatException) {
            Log.e("DiscoveryManager", "Failed to convert registKey to int", e)
            return
        }
        DiscoveryService.wakeup(discoveryService, host, credential, ps5)
    }

    private fun updateService() {
        if (active && !paused && discoveryService == null) {
            prevDiscoveredHost.clear()
            discoveredHostsSubjectRaw.onNext(listOf())
            try {
                discoveryService = DiscoveryService(
                    DiscoveryServiceOptions(
                        HOSTS_MAX, DROP_PINGS, PING_MS, InetSocketAddress("255.255.255.255", PORT)
                    ), discoveredHostsSubjectRaw::onNext
                )
            } catch (e: CreateError) {
                Log.e("DiscoveryManager", "Failed to start Discovery Service: $e")
            }
        } else if ((!active || paused) && discoveryService != null) {
            val service = discoveryService ?: return
            service.dispose()
            discoveryService = null
            if (!active)
                discoveredHostsSubjectRaw.onNext(listOf())
        }
    }
}