// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.base

import android.content.Intent
import android.content.pm.PackageManager
import android.os.Bundle
import androidx.annotation.CallSuper
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import androidx.lifecycle.LifecycleOwner
import androidx.viewbinding.ViewBinding
import co.vulcanlabs.psremote.R
import co.vulcanlabs.psremote.data.UserRepository
import co.vulcanlabs.psremote.eventbus.PermissionRequest
import co.vulcanlabs.psremote.eventbus.RxBus
import co.vulcanlabs.psremote.eventbus.RxBusListener
import co.vulcanlabs.psremote.management.DiscoveryManager
import co.vulcanlabs.psremote.utils.checkAppPermission
import co.vulcanlabs.psremote.utils.getViewBinding
import javax.inject.Inject

abstract class BaseActivity<T : ViewBinding>(private val clazz: Class<T>) : AppCompatActivity(),
    LifecycleOwner, RxBusListener {

    @Inject
    lateinit var discoveryManager: DiscoveryManager

    @Inject
    lateinit var userRepository: UserRepository

    lateinit var viewbinding: T

    private val permissionRequestCallBack = HashMap<Int, (result: Boolean) -> Unit>()

    var activityRequestCode = 1

    private var permissionRequestCode = 1

    val activityResult = HashMap<Int, (resultCode: Int, data: Intent?) -> Unit>()


    private val uuid by lazy { RxBus.INSTANCE.getNewUUID() }

    override fun getViewUUID() = uuid

    abstract fun setupView(savedInstanceState: Bundle?)

    @CallSuper
    @Suppress("DEPRECATION")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        overridePendingTransitionEnter()


        getViewBinding(layoutInflater, clazz).apply {
            viewbinding = this
            setContentView(root)
        }

        RxBus.INSTANCE.dataListen<PermissionRequest>(this) {
            requestPermission(it.permissionList, it.callback)
        }


        try {
            setupView(savedInstanceState)
        } catch (e: Exception) {
        }
    }

    open fun startActivityViewForResult(
        intent: Intent,
        callback: (resultCode: Int, data: Intent?) -> Unit
    ) {
        activityRequestCode = activityRequestCode % 1001 + 1
        startActivityForResult(intent, activityRequestCode)
        activityResult[activityRequestCode] = callback
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        activityResult[requestCode]?.invoke(resultCode, data)
        activityResult.remove(requestCode)
    }


    fun requestPermission(permissionList: List<String>, callback: (result: Boolean) -> Unit) {
        if (!checkAppPermission(permissionList)) {
            val requestCode = permissionRequestCode++
            ActivityCompat.requestPermissions(
                this, permissionList.toTypedArray(), requestCode
            )
            permissionRequestCallBack[requestCode] = callback
        } else {
            callback.invoke(true)
        }
    }

    override fun onRequestPermissionsResult(
        requestCode: Int, permissions: Array<out String>, grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        permissionRequestCallBack[requestCode]?.invoke(grantResults.all { it == PackageManager.PERMISSION_GRANTED })
        permissionRequestCallBack.remove(requestCode)
    }

    protected open fun isBottomUpActivity() = false

    override fun finish() {
        super.finish()
        overridePendingTransitionExit()
    }

    override fun onDestroy() {
        super.onDestroy()
        RxBus.INSTANCE.unRegister(this)
    }

    /**
     * Overrides the pending Activity transition by performing the "Enter" animation.
     */
    private fun overridePendingTransitionEnter() {
        if (isBottomUpActivity()) {
            overridePendingTransition(R.anim.slide_up, R.anim.stay)
        } else {
            overridePendingTransition(R.anim.slide_from_right, R.anim.slide_to_left)
        }
    }

    /**
     * Overrides the pending Activity transition by performing the "Exit" animation.
     */
    private fun overridePendingTransitionExit() {
        if (isBottomUpActivity()) {
            overridePendingTransition(R.anim.stay, R.anim.slide_down)
        } else {
            overridePendingTransition(R.anim.slide_from_left, R.anim.slide_to_right)
        }
    }

}