// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.discovery

import DiscoveryScreen
import android.content.Intent
import android.os.Bundle
import androidx.activity.viewModels
import androidx.compose.animation.ExperimentalAnimationApi
import androidx.compose.material.MaterialTheme
import androidx.lifecycle.lifecycleScope
import co.vulcanlabs.psremote.common.*
import co.vulcanlabs.psremote.data.Result
import co.vulcanlabs.psremote.databinding.ActivityDiscoveryBinding
import co.vulcanlabs.psremote.management.DeviceManager
import co.vulcanlabs.psremote.ui.base.BaseActivity
import co.vulcanlabs.psremote.ui.alertdialog.LoginSuccessDialogFragment
import co.vulcanlabs.psremote.ui.alertdialog.LogoutAlertDialogFragment
import co.vulcanlabs.psremote.ui.home.HomeActivity
import co.vulcanlabs.psremote.ui.registerdevice.RegisterActivity
import co.vulcanlabs.psremote.ui.registerdevice.RegisterViewModel
import co.vulcanlabs.psremote.ui.splash.SplashActivity
import co.vulcanlabs.psremote.utils.collectLatestWhenStarted
import co.vulcanlabs.psremote.utils.showDialog
import co.vulcanlabs.psremote.utils.showToast
import co.vulcanlabs.psremote.utils.startActivityView
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.delay

@AndroidEntryPoint
class DiscoveryActivity :
    BaseActivity<ActivityDiscoveryBinding>(ActivityDiscoveryBinding::class.java),
    LogoutAlertDialogFragment.OnLogoutConfirmListener {

    companion object {
        const val EXTRA_SHOW_LOGIN_SUCCESS = "EXTRA_SHOW_LOGIN_SUCCESS"
        const val EXTRA_AS_DEVICE_MANAGEMENT = "EXTRA_AS_DEVICE_MANAGEMENT"
    }

    private val viewModel: DiscoveryViewModel by viewModels()

    @ExperimentalAnimationApi
    override fun setupView(savedInstanceState: Bundle?) {
        renderView()
        if (intent.getBooleanExtra(EXTRA_SHOW_LOGIN_SUCCESS, false)) {
            lifecycleScope.launchWhenResumed {
                delay(300)
                LoginSuccessDialogFragment.newInstance()
                    .show(supportFragmentManager, LoginSuccessDialogFragment.TAG)
                intent.putExtra(EXTRA_SHOW_LOGIN_SUCCESS, false)
            }
        }

        viewModel.setAsDeviceManagement(intent.getBooleanExtra(EXTRA_AS_DEVICE_MANAGEMENT, false))

        viewModel.logOutEvent.collectLatestWhenStarted(lifecycleScope) {
            if (it is Result.Success) {
                DeviceManager.setCurrentDevice(null)
                startActivityView<SplashActivity> {
                    flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK
                }
            } else if (it is Result.Failure) {
                it.exception.message.orEmpty().showToast(false)
            }
        }
    }

    @ExperimentalAnimationApi
    private fun renderView() {
        viewbinding.composeView.setContent {
            MaterialTheme {
                DiscoveryScreen(
                    ::onDeviceClick,
                    ::onAddClick,
                    ::finish,
                    ::onLogoutClick
                )
            }
        }
    }

    private fun onLogoutClick() {
        showDialog(LogoutAlertDialogFragment.newInstance(), LogoutAlertDialogFragment.TAG)
    }

    override fun onLogoutConfirm() {
        viewModel.logOut()
    }

    private fun onAddClick() {
        startActivityView<RegisterActivity> {
            putExtra(
                RegisterActivity.EXTRA_SOUCE,
                RegisterViewModel.Source.Manually,
            )
        }
    }

    private fun onDeviceClick(device: DisplayHost) {
        val isRegistered = device.isRegistered && device.registeredHost != null
        if (isRegistered) {
            selectRegisteredDevice(device)
        } else {
            selectUnregisterDevice(device)
        }
    }

    private fun selectUnregisterDevice(device: DisplayHost) {
        // Register
        startActivityView<RegisterActivity> {
            putExtra(RegisterActivity.EXTRA_HOST, device.host)
            putExtra(RegisterActivity.EXTRA_SOUCE, RegisterViewModel.Source.Select)
            if (device is ManualDisplayHost)
                putExtra(RegisterActivity.EXTRA_ASSIGN_MANUAL_HOST_ID, device.manualHost.id)
        }
    }

    private fun selectRegisteredDevice(device: DisplayHost) {
        device.registeredHost ?: return
        DeviceManager.setCurrentDevice(device)
        startActivityView<HomeActivity> {
            if (viewModel.asDeviceManagement.value) {
                flags = Intent.FLAG_ACTIVITY_CLEAR_TOP
            }
        }
    }

}

