// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

import android.content.res.Configuration
import android.view.LayoutInflater
import androidx.compose.animation.Crossfade
import androidx.compose.animation.ExperimentalAnimationApi
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.*
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.rxjava3.subscribeAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalConfiguration
import androidx.compose.ui.res.colorResource
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.compose.ui.viewinterop.AndroidView
import androidx.lifecycle.viewmodel.compose.viewModel
import co.vulcanlabs.psremote.R
import co.vulcanlabs.psremote.common.DisplayHost
import co.vulcanlabs.psremote.ui.discovery.DeviceList
import co.vulcanlabs.psremote.ui.discovery.DiscoveryTopBar
import co.vulcanlabs.psremote.ui.discovery.DiscoveryViewModel

@ExperimentalAnimationApi
@Composable
fun DiscoveryScreen(
    onDeviceClick: (DisplayHost) -> Unit,
    onAddClick: () -> Unit,
    onBackClick: () -> Unit,
    onLogoutClick: () -> Unit
) {
    Column(
        Modifier
            .fillMaxSize()
            .background(colorResource(id = R.color.colorPrimary)),
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        DiscoveryTopBar(
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = 16.dp)
                .background(Color.Transparent),
            onBackClick,
            onLogoutClick
        )

        val viewModel: DiscoveryViewModel = viewModel()
        val devices = viewModel.displayHosts.subscribeAsState(initial = emptyList())

        Crossfade(
            targetState = devices.value, modifier = Modifier
                .padding(top = 10.dp)
                .fillMaxWidth()
                .weight(1f)
        ) {
            if (it.isNotEmpty()) {
                DeviceList(
                    modifier = Modifier
                        .fillMaxSize(),
                    onDeviceClick,
                    it
                )
            } else {
                Column(
                    modifier = Modifier
                        .fillMaxSize(),
                    verticalArrangement = Arrangement.Center,
                    horizontalAlignment = Alignment.CenterHorizontally
                ) {
                    Image(
                        painter = painterResource(id = R.drawable.ic_device),
                        contentDescription = ""
                    )
                    Text(
                        stringResource(R.string.discovery_instruction),
                        fontSize = 18.sp,
                        color = Color.White,
                        modifier = Modifier
                            .padding(top = 23.dp)
                            .wrapContentSize()
                            .padding(horizontal = 10.dp),
                        textAlign = TextAlign.Center
                    )

                }
            }
        }

        // Add manually

        Text(
            text = stringResource(R.string.cant_find_your_device),
            modifier = Modifier.wrapContentSize(),
            color = Color.White,
            fontSize = 14.sp
        )

        AndroidView(
            modifier = Modifier
                .wrapContentHeight()
                .fillMaxWidth()
                .padding(
                    start = 16.dp,
                    end = 16.dp,
                    top = 10.dp,
                    bottom = if (LocalConfiguration.current.orientation == Configuration.ORIENTATION_PORTRAIT) 40.dp else 5.dp
                ),
            factory = { context ->
                LayoutInflater.from(context).inflate(R.layout.layout_button_add_manually, null)
                    .apply {
                        setOnClickListener {
                            onAddClick()
                        }
                    }
            }
        )
    }
}