// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.discovery

import android.app.Application
import androidx.lifecycle.viewModelScope
import co.vulcanlabs.psremote.common.DiscoveredDisplayHost
import co.vulcanlabs.psremote.common.ManualDisplayHost
import co.vulcanlabs.psremote.common.ManualHost
import co.vulcanlabs.psremote.data.DeviceRepository
import co.vulcanlabs.psremote.data.Result
import co.vulcanlabs.psremote.data.UserRepository
import co.vulcanlabs.psremote.management.DiscoveryManager
import co.vulcanlabs.psremote.management.serverMac
import co.vulcanlabs.psremote.ui.base.BaseViewModel
import dagger.hilt.android.lifecycle.HiltViewModel
import io.reactivex.rxjava3.kotlin.Observables
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asSharedFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class DiscoveryViewModel @Inject constructor(
    app: Application,
    private val deviceRepository: DeviceRepository,
    private val discoveryManager: DiscoveryManager,
    private val userRepository: UserRepository
) :
    BaseViewModel(app) {

    private val _isRefreshingState = MutableStateFlow(true)
    val isRefreshingState = _isRefreshingState.asStateFlow()

    private val _logOutEvent = MutableSharedFlow<Result<Unit>>()
    val logOutEvent = _logOutEvent.asSharedFlow()

    val displayHosts by lazy {
        Observables.combineLatest(
            deviceRepository.getAllManualHost().toObservable(),
            deviceRepository.getAllRegisteredHost().toObservable(),
            discoveryManager.discoveredHosts
        )
        { manualHosts, registeredHosts, discoveredHosts ->
            val macRegisteredHosts = registeredHosts.associateBy { it.serverMac }
            val idRegisteredHosts = registeredHosts.associateBy { it.id }
            val discoveredDisplayHost = discoveredHosts.map {
                DiscoveredDisplayHost(it.serverMac?.let { mac -> macRegisteredHosts[mac] }, it)
            }

            val manualDisplayHosts = manualHosts.map {
                ManualDisplayHost(
                    it.registeredHost?.let { id -> idRegisteredHosts[id] },
                    it
                )
            }
            discoveredDisplayHost + manualDisplayHosts.filter { manual ->
                manual.isRegistered && discoveredDisplayHost.all { discovered -> discovered.host != manual.host }
            }
        }
    }

    private val _asDeviceManagement = MutableStateFlow(false)
    val asDeviceManagement = _asDeviceManagement.asStateFlow()

    fun deleteManualHost(manualHost: ManualHost) = viewModelScope.launch {
        deviceRepository.deleteManualHost(manualHost)
    }

    override fun onCleared() {
        super.onCleared()
        discoveryManager.dispose()
    }

    fun refreshDiscovery() {
        _isRefreshingState.value = true
        discoveryManager.refresh()
    }

    fun doneRefreshing() {
        _isRefreshingState.value = false
    }

    fun setAsDeviceManagement(asDeviceManagement: Boolean) {
        _asDeviceManagement.value = asDeviceManagement
    }

    fun logOut() = viewModelScope.launch {
        when (val logOutResult = userRepository.logOut()) {
            is Result.Failure -> _logOutEvent.emit(logOutResult)
            is Result.Success -> _logOutEvent.emit(deviceRepository.clearAllDevices())
        }
    }

}