// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.home

import android.content.Intent
import android.os.Bundle
import androidx.activity.viewModels
import androidx.core.view.isGone
import androidx.core.view.isVisible
import co.vulcanlabs.psremote.BuildConfig
import co.vulcanlabs.psremote.R
import co.vulcanlabs.psremote.databinding.ActivityHomeBinding
import co.vulcanlabs.psremote.management.DeviceManager
import co.vulcanlabs.psremote.ui.alertdialog.SignInAlertDialogFragment
import co.vulcanlabs.psremote.ui.base.BaseActivity
import co.vulcanlabs.psremote.ui.discovery.DiscoveryActivity
import co.vulcanlabs.psremote.ui.play.PlayActivity
import co.vulcanlabs.psremote.ui.play.PlayViewModel
import co.vulcanlabs.psremote.ui.profile.ProfileSettingActivity
import co.vulcanlabs.psremote.utils.showDialog
import co.vulcanlabs.psremote.utils.startActivityView
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class HomeActivity : BaseActivity<ActivityHomeBinding>(ActivityHomeBinding::class.java) {

    companion object {
        const val EXTRA_IS_IN_EXHIBITION_MODE = "EXTRA_IS_IN_EXHIBITION_MODE"
    }

    private val viewModel: HomeViewModel by viewModels()

    override fun setupView(savedInstanceState: Bundle?) {
        setListeners()

        bindIntent(intent, savedInstanceState)

    }

    private fun bindIntent(intent: Intent?, savedInstanceState: Bundle?) {
        viewModel.isInExhibitionMode = (intent?.getBooleanExtra(EXTRA_IS_IN_EXHIBITION_MODE, false)
            ?: false)

        if (!viewModel.didLogin()) {
            setUpExhibitionMode()
        } else {
            viewbinding.navView.txtDrawerTitle.text = DeviceManager.getCurrentDevice()?.name
            viewbinding.navView.txtDrawerSubtitle.text = viewModel.getCurrentUserPsnId()
        }

        viewbinding.txtTitle.run {
            text = DeviceManager.getCurrentDevice()?.name
            isVisible = !text.isNullOrEmpty()
        }
        viewbinding.txtVersionInfo.run {
            text = getString(
                R.string.version_info_format,
                BuildConfig.VERSION_NAME,
                BuildConfig.VERSION_CODE
            )
            isVisible = BuildConfig.DEBUG
        }
    }

    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        bindIntent(intent, null)
    }

    private fun setUpExhibitionMode() {
        viewbinding.navView.txtDrawerTitle.text = getString(R.string.sign_in_to_your_ps_account)
        viewbinding.navView.txtDrawerSubtitle.text = getString(R.string.to_access_gameplay)
        viewbinding.navView.txtProfileSetting.isGone = true
        viewbinding.navView.txtDeviceManagement.isGone = true
        with(viewbinding.navView.txtLogin) {
            isVisible = true
            setOnClickListener {
                showDialogLoginAlert()
            }
        }
        viewbinding.btnSignIn.isVisible = true
        viewbinding.btnSignIn.setOnClickListener {
            showDialogLoginAlert()
        }

    }

    private fun showDialogLoginAlert() {
        showDialog(
            SignInAlertDialogFragment.newInstance(),
            SignInAlertDialogFragment.TAG
        )
    }

    private fun setListeners() {
        viewbinding.txtRemote.setOnClickListener {
            openPlayScreen(PlayViewModel.Mode.Remote)
        }

        viewbinding.txtGamepad.setOnClickListener {
            openPlayScreen(PlayViewModel.Mode.Gamepad)
        }

        viewbinding.imgMore.setOnClickListener {
            viewbinding.root.open()
        }

        viewbinding.navView.imgBack.setOnClickListener {
            viewbinding.root.close()
        }
        viewbinding.navView.txtDeviceManagement.setOnClickListener {
            startActivityView<DiscoveryActivity> {
                putExtra(DiscoveryActivity.EXTRA_SHOW_LOGIN_SUCCESS, false)
                putExtra(DiscoveryActivity.EXTRA_AS_DEVICE_MANAGEMENT, true)
            }
        }
        viewbinding.navView.txtProfileSetting.setOnClickListener {
            startActivityView<ProfileSettingActivity> { }
        }

    }

    private fun openPlayScreen(mode: PlayViewModel.Mode) {
        if (!viewModel.didLogin()) {
            startActivityView<PlayActivity> {
                putExtra(PlayActivity.EXTRA_IS_IN_EXHIBITION_MODE, viewModel.isInExhibitionMode)
                putExtra(PlayActivity.EXTRA_MODE, mode)
            }

        } else {
            val currentDevice = DeviceManager.getCurrentDevice() ?: return
            val connectInfo = currentDevice.toConnectInfo(viewModel.getVideoProfile())
            startActivityViewForResult(Intent(this, PlayActivity::class.java).apply {
                putExtra(PlayActivity.EXTRA_CONNECT_INFO, connectInfo)
                putExtra(PlayActivity.EXTRA_MODE, mode)
                putExtra(PlayActivity.EXTRA_IS_IN_EXHIBITION_MODE, viewModel.isInExhibitionMode)

            }) { _, _ -> }
        }
    }

    override fun onBackPressed() {
        val drawer = viewbinding.root
        if (drawer.isOpen) {
            drawer.close()
        } else {
            super.onBackPressed()
        }
    }
}