// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.play

import android.annotation.SuppressLint
import android.content.pm.ActivityInfo
import android.content.res.Configuration
import android.os.Build
import android.os.Bundle
import android.os.VibrationEffect
import android.os.Vibrator
import android.view.*
import androidx.activity.viewModels
import androidx.constraintlayout.widget.ConstraintSet
import androidx.core.content.getSystemService
import androidx.core.view.*
import androidx.fragment.app.Fragment
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.lifecycleScope
import co.vulcanlabs.psremote.R
import co.vulcanlabs.psremote.chiaki.session.*
import co.vulcanlabs.psremote.common.*
import co.vulcanlabs.psremote.databinding.ActivityPlayBinding
import co.vulcanlabs.psremote.ui.base.BaseActivity
import co.vulcanlabs.psremote.ui.alertdialog.DialogInPlayScreen
import co.vulcanlabs.psremote.ui.alertdialog.PinRequestDialogFragment
import co.vulcanlabs.psremote.ui.alertdialog.PlayInterruptDialogFragment
import co.vulcanlabs.psremote.ui.alertdialog.SignInAlertDialogFragment
import co.vulcanlabs.psremote.ui.touchcontrols.DefaultTouchControlsFragment
import co.vulcanlabs.psremote.ui.touchcontrols.TouchControlsFragment
import co.vulcanlabs.psremote.ui.touchcontrols.TouchpadOnlyFragment
import co.vulcanlabs.psremote.utils.*
import dagger.hilt.android.AndroidEntryPoint
import io.reactivex.rxjava3.disposables.CompositeDisposable
import io.reactivex.rxjava3.kotlin.addTo
import kotlinx.coroutines.Job
import kotlinx.coroutines.delay
import javax.inject.Inject
import kotlin.math.min


@AndroidEntryPoint
class PlayActivity : BaseActivity<ActivityPlayBinding>(ActivityPlayBinding::class.java) {

    companion object {
        const val EXTRA_CONNECT_INFO = "EXTRA_CONNECT_INFO"
        const val EXTRA_MODE = "EXTRA_MODE"
        const val EXTRA_IS_IN_EXHIBITION_MODE = "EXTRA_IS_IN_EXHIBITION_MODE"
    }

    private val controlsDisposable = CompositeDisposable()

    private val viewModel: PlayViewModel by viewModels()

    private var hideControllerJob: Job? = null

    private val listPointerIds = mutableSetOf<Int>()

    private var currentOrientation: Int? = null

    @Inject
    lateinit var quitMessageFactory: ErrorQuitMessageFactory

    @SuppressLint("SourceLockedOrientationActivity")
    override fun setupView(savedInstanceState: Bundle?) {

        viewModel.mode = (intent?.getSerializableExtra(EXTRA_MODE) as? PlayViewModel.Mode)
            ?: PlayViewModel.Mode.Remote
        viewModel.isInExhibitionMode =
            (intent?.getBooleanExtra(EXTRA_IS_IN_EXHIBITION_MODE, false)
                ?: false)

        if (viewModel.isInExhibitionMode) {
            setUpForExhibitionMode()
        }

        viewModel.getLocalPlaySetting()

        viewModel.setConnectInfo(intent.getParcelableExtra(EXTRA_CONNECT_INFO))
        if (viewModel.mode == PlayViewModel.Mode.Remote) {
            viewModel.session?.attachToSurfaceView(viewbinding.surfaceView)
        } else {
            viewbinding.aspectRatioLayout.isGone = true
        }
        viewModel.state?.observe(this, ::stateChanged)
        adjustSurfaceViewAspect()

        handleOrientationChange(resources.configuration.orientation)

        observeObservables()

        setListeners()
    }

    private fun setListeners() {
        viewbinding.root.setOnClickListener {
            viewbinding.controlsFragment.isVisible = true
            checkHideControls()
        }

        viewbinding.layoutExhibitionWarning.btnLogin.setOnClickListener {
            showDialog(
                SignInAlertDialogFragment.newInstance(),
                SignInAlertDialogFragment.TAG
            )

        }
    }

    private fun observeObservables() {
        viewModel.lockOrientationMode.collectWhenStarted(lifecycleScope) {
            when (it) {
                PlayViewModel.LockOrientationMode.AUTO_ROTATE -> requestedOrientation =
                    ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED
                PlayViewModel.LockOrientationMode.LOCK_LANDSCAPE -> requestedOrientation =
                    ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE
                PlayViewModel.LockOrientationMode.LOCK_PORTRAIT -> requestedOrientation =
                    ActivityInfo.SCREEN_ORIENTATION_PORTRAIT
            }
        }

        viewModel.alwaysShowController.collectWhenStarted(lifecycleScope) { isAlways ->
            if (isAlways == null) {
                return@collectWhenStarted
            }
            if (isAlways) {
                viewbinding.controlsFragment.isVisible = true
                hideControllerJob?.cancel()
                hideControllerJob = null
            } else {
                checkHideControls()
            }
        }

        viewModel.disconnectEvent.collectWhenStarted(lifecycleScope) {
            if (!viewModel.isInExhibitionMode) {
                setResult(RESULT_OK)
            }
            finish()
        }

        viewModel.session?.rumbleState?.observe(this) {
            vibrateRumble(it.left, it.right)
        }
    }

    private fun vibrateRumble(left: UByte, right: UByte) {
        val vibrator = getSystemService<Vibrator>() ?: return
        val amplitude = min(255, (left.toInt() + right.toInt()) / 2)
        vibrator.cancel()
        if (amplitude == 0) {
            return
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            vibrator.vibrate(VibrationEffect.createOneShot(1000, amplitude))
        } else {
            vibrator.vibrate(1000)
        }
    }

    private fun setUpForExhibitionMode() {
        viewbinding.surfaceView.isGone = true
        viewbinding.layoutExhibitionWarning.root.isVisible = true
    }

    private fun checkHideControls() {
        if (listPointerIds.isNotEmpty()) {
            return
        }
        hideControllerJob?.cancel()
        if (viewModel.alwaysShowController.value == false && viewbinding.controlsFragment.isVisible) {
            hideControllerJob = lifecycleScope.launchWhenStarted {
                delay(3000)
                viewbinding.controlsFragment.isInvisible = true
            }
        }
    }

    private fun adjustSurfaceViewAspect() {
        val videoProfile = viewModel.session?.connectInfo?.videoProfile ?: return
        viewbinding.aspectRatioLayout.setProperties(
            videoProfile.width.toFloat() / videoProfile.height.toFloat(),
            if (currentOrientation == Configuration.ORIENTATION_LANDSCAPE) {
                Gravity.CENTER
            } else {
                Gravity.CENTER_HORIZONTAL or Gravity.TOP
            }
        )
    }

    override fun onResume() {
        super.onResume()
        viewModel.resume()
    }


    override fun onWindowFocusChanged(hasFocus: Boolean) {
        super.onWindowFocusChanged(hasFocus)
        if (hasFocus) {
            hideNavigationAndStatusBar()
        }
    }

    override fun onPause() {
        super.onPause()
        viewModel.pause()
    }

    override fun onAttachFragment(fragment: Fragment) {
        super.onAttachFragment(fragment)
        if (fragment is TouchControlsFragment) {
            fragment.controllerState
                .subscribe { viewModel.input.touchControllerState = it }
                .addTo(controlsDisposable)
//            fragment.onScreenControlsEnabled = viewModel.onScreenControlsEnabled
            if (fragment is TouchpadOnlyFragment) {
//                fragment.touchpadOnlyEnabled = viewModel.touchpadOnlyEnabled
                fragment.touchpadOnlyEnabled = MutableLiveData(false)
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        controlsDisposable.dispose()
        discoveryManager.onGameChange = null
    }

    private fun hideNavigationAndStatusBar() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            window.setDecorFitsSystemWindows(false)
            window.insetsController?.run {
                hide(WindowInsets.Type.statusBars() or WindowInsets.Type.navigationBars())
                systemBarsBehavior =
                    WindowInsetsController.BEHAVIOR_SHOW_TRANSIENT_BARS_BY_SWIPE
            }
        } else {
            window.decorView.systemUiVisibility = (View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY
                    or View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                    or View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                    or View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                    or View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    or View.SYSTEM_UI_FLAG_FULLSCREEN)
        }
    }

    override fun dispatchTouchEvent(ev: MotionEvent?): Boolean {
        when (ev?.actionMasked) {
            MotionEvent.ACTION_DOWN, MotionEvent.ACTION_POINTER_DOWN -> {
                hideControllerJob?.cancel()
                listPointerIds.add(ev.getPointerId(ev.actionIndex))
            }

            MotionEvent.ACTION_UP, MotionEvent.ACTION_POINTER_UP -> {
                listPointerIds.remove(ev.getPointerId(ev.actionIndex))
                checkHideControls()
            }
        }

        return super.dispatchTouchEvent(ev)
    }

    override fun dispatchKeyEvent(event: KeyEvent) =
        viewModel.input.dispatchKeyEvent(event) || super.dispatchKeyEvent(event)

    override fun onGenericMotionEvent(event: MotionEvent) =
        viewModel.input.onGenericMotionEvent(event) || super.onGenericMotionEvent(event)

    private fun stateChanged(state: StreamState) {
        when (state) {
            StreamStateConnecting -> {
                val showLoading = !viewModel.isInExhibitionMode
                if (viewbinding.loading.isVisible != showLoading) {
                    viewbinding.loading.isVisible = showLoading
                }
            }
            is StreamStateQuit -> {
                if (state.reason.isStopped) {
                    return
                }
                showDialog(
                    PlayInterruptDialogFragment.newInstance(quitMessageFactory.create(state.reason)),
                    PlayInterruptDialogFragment.TAG
                )
            }

            is StreamStateLoginPinRequest -> {
                showDialog(PinRequestDialogFragment.newInstance(), PinRequestDialogFragment.TAG)
            }
            else -> {
                viewbinding.loading.isGone = true
            }
        }
    }

    override fun onConfigurationChanged(newConfig: Configuration) {
        super.onConfigurationChanged(newConfig)
        handleOrientationChange(newConfig.orientation)
    }

    private fun handleOrientationChange(orientation: Int) {
        if (orientation == currentOrientation) {
            return
        }

        currentOrientation = orientation

        relayout(orientation)

        adjustSurfaceViewAspect()

        supportFragmentManager.beginTransaction()
            .replace(R.id.controlsFragment, DefaultTouchControlsFragment())
            .commitNowAllowingStateLoss()

        restoreDialogs()
    }

    private fun restoreDialogs() {
        supportFragmentManager.run {
            fragments.filter { it.isAdded && !it.isRemoving }
                .filterIsInstance(DialogInPlayScreen::class.java)
                .forEach {
                    it.onOrientationChange()
                }
        }
    }

    private fun relayout(orientation: Int) {
        val constraintSet = ConstraintSet().apply {
            clone(viewbinding.root)
        }
        if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
            constraintSet.connect(
                R.id.aspectRatioLayout,
                ConstraintSet.TOP,
                ConstraintSet.PARENT_ID,
                ConstraintSet.TOP
            )
            constraintSet.connect(
                R.id.aspectRatioLayout,
                ConstraintSet.BOTTOM,
                ConstraintSet.PARENT_ID,
                ConstraintSet.BOTTOM
            )
            constraintSet.connect(
                R.id.controlsFragment,
                ConstraintSet.TOP,
                ConstraintSet.PARENT_ID,
                ConstraintSet.TOP
            )
            constraintSet.connect(
                R.id.controlsFragment,
                ConstraintSet.BOTTOM,
                ConstraintSet.PARENT_ID,
                ConstraintSet.BOTTOM
            )
            constraintSet.connect(
                R.id.layoutExhibitionWarning,
                ConstraintSet.TOP,
                R.id.controlsFragment,
                ConstraintSet.TOP
            )

            constraintSet.connect(
                R.id.layoutExhibitionWarning,
                ConstraintSet.BOTTOM,
                R.id.controlsFragment,
                ConstraintSet.BOTTOM
            )

        } else {
            constraintSet.connect(
                R.id.aspectRatioLayout,
                ConstraintSet.TOP,
                ConstraintSet.PARENT_ID,
                ConstraintSet.TOP
            )
            constraintSet.connect(
                R.id.aspectRatioLayout,
                ConstraintSet.BOTTOM,
                R.id.horizontalGuideline,
                ConstraintSet.BOTTOM
            )
            constraintSet.connect(
                R.id.controlsFragment,
                ConstraintSet.TOP,
                R.id.aspectRatioLayout,
                ConstraintSet.BOTTOM
            )
            constraintSet.connect(
                R.id.controlsFragment,
                ConstraintSet.BOTTOM,
                ConstraintSet.PARENT_ID,
                ConstraintSet.BOTTOM
            )

            constraintSet.connect(
                R.id.layoutExhibitionWarning,
                ConstraintSet.TOP,
                R.id.aspectRatioLayout,
                ConstraintSet.TOP
            )

            constraintSet.connect(
                R.id.layoutExhibitionWarning,
                ConstraintSet.BOTTOM,
                R.id.aspectRatioLayout,
                ConstraintSet.BOTTOM
            )
        }
        constraintSet.applyTo(viewbinding.root)
        viewbinding.layoutExhibitionWarning.root.run {
            updatePadding(bottom = if (orientation == Configuration.ORIENTATION_LANDSCAPE) 120.dpToPx() else 0)
            setBackgroundResource(if (orientation == Configuration.ORIENTATION_LANDSCAPE) R.color.transparent else R.color.colorPrimary)
        }
    }
}