// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.play

import android.app.Application
import androidx.lifecycle.LiveData
import androidx.lifecycle.Transformations
import androidx.lifecycle.viewModelScope
import co.vulcanlabs.psremote.chiaki.ConnectInfo
import co.vulcanlabs.psremote.chiaki.session.*
import co.vulcanlabs.psremote.common.*
import co.vulcanlabs.psremote.data.local.Preferences
import co.vulcanlabs.psremote.ui.base.BaseViewModel
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import java.util.logging.LogManager
import javax.inject.Inject

@HiltViewModel
class PlayViewModel @Inject constructor(
    app: Application,
    private val preferences: Preferences
) : BaseViewModel(app) {

    var isInExhibitionMode = false

    private var isPause = false

    private val _lockOrientationMode = MutableStateFlow<LockOrientationMode?>(null)
    val lockOrientationMode = _lockOrientationMode.asStateFlow()

    private val _alwaysShowController = MutableStateFlow<Boolean?>(null)
    val alwaysShowController = _alwaysShowController.asStateFlow()

    private val _disconnectEvent = MutableSharedFlow<Unit>()
    val disconnectEvent = _disconnectEvent.asSharedFlow()

    enum class LockOrientationMode(val value: String) {
        AUTO_ROTATE("AUTO_ROTATE"), LOCK_PORTRAIT("LOCK_PORTRAIT"), LOCK_LANDSCAPE("LOCK_LANDSCAPE");

        companion object {
            fun from(value: String) = when (value) {
                AUTO_ROTATE.value -> AUTO_ROTATE
                LOCK_PORTRAIT.value -> LOCK_PORTRAIT
                LOCK_LANDSCAPE.value -> LOCK_LANDSCAPE
                else -> AUTO_ROTATE
            }
        }
    }

    companion object {
        private const val MAX_RETRY_TIME = 5
        private const val RETRY_INTERVAL_MS = 1000L
    }

    enum class BottomBarState {
        OPEN, CLOSE
    }

    var mode: Mode = Mode.Remote

    enum class Mode {
        Remote, Gamepad
    }

    private var _session: StreamSession? = null
    val input = StreamInput(app)
    val session
        get() = _session

    private var isReconnecting = false

    var state: LiveData<StreamState>? = null

    private var retryTime = 0

    private val _connectInfoState: MutableStateFlow<ConnectInfo?> = MutableStateFlow(null)

    var bottomBarState: BottomBarState = BottomBarState.OPEN

    fun setConnectInfo(connectInfo: ConnectInfo?) {
        retryTime = 0
        if (connectInfo == _connectInfoState.value) {
            return
        }
        _connectInfoState.value = connectInfo
        connectInfo?.let {
            _session = StreamSession(
                it,
                LogManager.getLogManager(),
                true,
                input,
                mode == Mode.Gamepad
            )
            initStateLiveData()
        }
    }

    private fun initStateLiveData() {
        state = _session?.state?.run {
            Transformations.map(this) {
                if (it is StreamStateQuit && !it.reason.isStopped && retryTime <= MAX_RETRY_TIME) {
                    retryTime++
                    viewModelScope.launch {
                        delay(RETRY_INTERVAL_MS)
                        reconnect()
                    }
                    StreamStateConnecting
                } else if (isReconnecting && it == StreamStateIdle) {
                    StreamStateConnecting
                } else {
                    if (it !is StreamStateConnecting) {
                        isReconnecting = false
                        // User reconnect
                    }
                    it
                }
            }
        }
    }

    fun getLocalPlaySetting() = viewModelScope.launch(Dispatchers.IO) {
        if (mode == Mode.Remote) {
            _lockOrientationMode.emit(LockOrientationMode.from(preferences.playSettingOrientationLockMode))
            _alwaysShowController.emit(preferences.playSettingAlwaysShowController)
        } else {
            _lockOrientationMode.emit(LockOrientationMode.LOCK_LANDSCAPE)
            _alwaysShowController.emit(true)
        }
    }

    fun setLockOrientationMode(lockOrientationMode: LockOrientationMode) =
        viewModelScope.launch(Dispatchers.IO) {
            if (lockOrientationMode == _lockOrientationMode.value) {
                return@launch
            }
            if (!isInExhibitionMode) {
                preferences.playSettingOrientationLockMode = lockOrientationMode.value
                // Re-fetch setting
                getLocalPlaySetting()
            } else {
                _lockOrientationMode.emit(lockOrientationMode)
            }
        }

    fun setAlwaysShowController(isAlways: Boolean) = viewModelScope.launch(Dispatchers.IO) {
        if (isAlways == alwaysShowController.value) {
            return@launch
        }
        if (!isInExhibitionMode) {
            preferences.playSettingAlwaysShowController = isAlways
            // Re-fetch setting
            getLocalPlaySetting()
        } else {
            _alwaysShowController.emit(isAlways)
        }
    }

    fun disconnect() = viewModelScope.launch {
        _disconnectEvent.emit(Unit)
    }

    fun reconnect() {
        if (isPause) {
            return
        }
        isReconnecting = true
        session?.run {
            shutdown()
            resume()
        }
    }

    fun setPin(pin: String) {
        session?.setLoginPin(pin)
    }

    fun pause() {
        isPause = true
        retryTime = 0
        _session?.pause()
    }

    fun resume() {
        isPause = false
        _session?.resume()
    }

}