// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.registerdevice

import android.os.Bundle
import androidx.core.view.isVisible
import androidx.core.widget.doAfterTextChanged
import androidx.fragment.app.viewModels
import androidx.lifecycle.lifecycleScope
import co.vulcanlabs.psremote.R
import co.vulcanlabs.psremote.common.ConsoleVersion
import co.vulcanlabs.psremote.databinding.FragmentRegisterBinding
import co.vulcanlabs.psremote.ui.base.BaseFragment
import co.vulcanlabs.psremote.ui.registerdevice.selectfirmware.FirmwareSelectDialogFragment
import co.vulcanlabs.psremote.utils.*
import dagger.hilt.android.AndroidEntryPoint


@AndroidEntryPoint
class RegisterFragment :
    BaseFragment<FragmentRegisterBinding>(FragmentRegisterBinding::class.java),
    FirmwareSelectDialogFragment.OnDialogInteractionListener {

    companion object {
        fun newInstance(arg: Bundle) = RegisterFragment().apply {
            arguments = arg
        }
    }

    private val viewModel: RegisterViewModel by viewModels()

    private fun checkFocusedToShowTutorial() {
        viewbinding?.root?.post {
            val videoRawId = when {
                viewbinding?.edtLayoutIp?.editText?.isFocused == true -> R.raw.tutorial_ip_software
                viewbinding?.edtLayoutRegisterNumber?.editText?.isFocused == true -> R.raw.tutorial_register_number
                else -> if (viewModel.registerSource == RegisterViewModel.Source.Select) R.raw.tutorial_system_software else R.raw.tutorial_ip_software
            }
            viewModel.setTutorialVideoId(videoRawId)
        }
    }

    override fun setupView(savedInstanceState: Bundle?) {
        viewModel.registerSource =
            (arguments?.getSerializable(RegisterActivity.EXTRA_SOUCE) as? RegisterViewModel.Source)
                ?: RegisterViewModel.Source.Manually
        setUpViewFromSource(viewModel.registerSource)
        val passedHost = arguments?.getString(RegisterActivity.EXTRA_HOST).orEmpty()
        if (passedHost.isNotEmpty()) {
            viewModel.setIp(passedHost)
        }

        viewModel.screenStateFlow.collectLatestWhenStarted(viewLifecycleOwner.lifecycleScope) {
            bindScreenState(it)
        }


        viewModel.connectState.collectLatestWhenStarted(viewLifecycleOwner.lifecycleScope) {
            bindConnectState(it)
        }

        viewModel.tutorialVideoId.collectLatestWhenStarted(viewLifecycleOwner.lifecycleScope) {
            if (it != 0) {
                viewbinding?.videoView?.setUp(it)
            }
        }

        setListeners()
    }

    override fun onStart() {
        super.onStart()
        checkFocusedToShowTutorial()
    }

    private fun setListeners() {
        viewbinding?.run {
            firmwareClickableView.setOnClickListener {
                showFirmwareSelectDialog()
            }

            imgBack.setOnClickListener {
                activity?.finish()
            }

            edtLayoutRegisterNumber.editText?.doAfterTextChanged {
                val text = it?.toString().orEmpty()
                viewModel.setRegisterNumber(text.toIntOrNull())
            }

            edtLayoutIp.editText?.doAfterTextChanged {
                val text = it?.toString().orEmpty()
                viewModel.setIp(text)
            }

            btnRegister.setOnClickListener {
                viewModel.register()
            }

            edtLayoutRegisterNumber.editText?.setOnFocusChangeListener { _, _ ->
                checkFocusedToShowTutorial()
            }

            edtLayoutIp.editText?.setOnFocusChangeListener { _, _ ->
                checkFocusedToShowTutorial()
            }

            root.setOnClickListener {
                it.hideKeyboard()
            }
        }

    }

    private fun showFirmwareSelectDialog() {
        val dialog =
            FirmwareSelectDialogFragment.newInstance(viewModel.screenStateFlow.value.firmWare)
                .apply {
                }
        showDialog(dialog, FirmwareSelectDialogFragment.TAG)
        viewModel.setTutorialVideoId(if (viewModel.registerSource == RegisterViewModel.Source.Select) R.raw.tutorial_system_software else R.raw.tutorial_ip_software)
    }

    private fun bindConnectState(state: RegisterViewModel.ConnectState) {
        when (state) {
            is RegisterViewModel.ConnectState.Connected -> {
                hideAppLoading()
                "Register new device successfully!".showToast(false)
                activity?.finish()
            }

            is RegisterViewModel.ConnectState.Connecting -> showAppLoading()

            is RegisterViewModel.ConnectState.Failure -> {
                hideAppLoading()
                when {
                    state.cause is RegistException -> {
                        val message = when (state.cause) {
                            is RegistException.InvalidInputException -> {
                                "Invalid input"
                            }
                            is RegistException.ConnectionCancelException -> {
                                "Connection canceled"
                            }
                            else -> {
                                "Connection failed"
                            }
                        }
                        message.showToast(false)
                    }
                    state.cause.message.isNullOrEmpty() -> {
                        "Unknown exception".showToast(false)
                    }
                    else -> {
                        state.cause.message.takeIf { message -> message.isNullOrEmpty() }
                            ?.showToast(false)
                    }
                }
            }

            is RegisterViewModel.ConnectState.Initial -> {
                // Nothing
            }
        }
    }

    private fun bindScreenState(state: RegisterViewModel.ScreenState) {
        if (state.registerNumber?.toString()
                .orEmpty() != viewbinding?.edtLayoutRegisterNumber?.editText?.text?.toString()
                .orEmpty()
        ) {
            viewbinding?.edtLayoutRegisterNumber?.editText?.setText(
                state.registerNumber?.toString().orEmpty()
            )
        }

        if (state.ip != viewbinding?.edtLayoutIp?.editText?.text?.toString().orEmpty()) {
            viewbinding?.edtLayoutIp?.editText?.setText(state.ip)
        }

        if (state.firmWare.string != viewbinding?.edtLayoutFirmware?.editText?.text?.toString()
                .orEmpty()
        ) {
            viewbinding?.edtLayoutFirmware?.editText?.setText(state.firmWare.string)
        }

        viewbinding?.btnRegister?.isEnabled =
            state.ip.isNotEmpty() && state.registerNumber != null
    }

    private fun setUpViewFromSource(source: RegisterViewModel.Source) {
        viewbinding?.edtLayoutIp?.isVisible = RegisterViewModel.Source.Manually == source
        viewbinding?.txtInstruction?.text = if (RegisterViewModel.Source.Manually == source) {
            getString(R.string.manually_add_device_message)
        } else {
            getString(R.string.enter_register_number_message)
        }
    }

    override fun onFirmwareSelected(consoleVersion: ConsoleVersion) {
        viewModel.setFirmware(consoleVersion)
    }

    override fun onDialogDismiss() {
        checkFocusedToShowTutorial()
    }
}