// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.registerdevice

import android.app.Application
import androidx.annotation.RawRes
import androidx.lifecycle.viewModelScope
import co.vulcanlabs.psremote.chiaki.*
import co.vulcanlabs.psremote.common.*
import co.vulcanlabs.psremote.data.DeviceRepository
import co.vulcanlabs.psremote.data.Result
import co.vulcanlabs.psremote.data.UserRepository
import co.vulcanlabs.psremote.ui.base.BaseViewModel
import co.vulcanlabs.psremote.utils.showLog
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch
import java.math.BigInteger
import javax.inject.Inject

@HiltViewModel
class RegisterViewModel @Inject constructor(
    app: Application,
    private val userRepository: UserRepository,
    private val deviceRepository: DeviceRepository
) : BaseViewModel(app) {

    private val _screenStateFlow = MutableStateFlow(ScreenState(null, "", ConsoleVersion.PS4_GE_8))
    val screenStateFlow = _screenStateFlow.asStateFlow()

    var registerSource: Source = Source.Manually

    private val _tutorialVideoId = MutableStateFlow(0)
    var tutorialVideoId = _tutorialVideoId.asStateFlow()

    fun setRegisterNumber(registerNumber: Int?) {
        val currentState = getCurrentState()
        if (registerNumber == currentState.registerNumber) {
            return
        }
        _screenStateFlow.value = currentState.copy(registerNumber = registerNumber)
    }

    fun setIp(ip: String) {
        val currentState = getCurrentState()
        if (ip == currentState.ip) {
            return
        }
        _screenStateFlow.value = currentState.copy(ip = ip)
    }

    fun setFirmware(firmWare: ConsoleVersion) {
        val currentState = getCurrentState()
        if (firmWare == currentState.firmWare) {
            return
        }
        _screenStateFlow.value = currentState.copy(firmWare = firmWare)
    }

    private fun getCurrentState() = _screenStateFlow.value

    fun register() {
        val currentState = getCurrentState()
        if (currentState.ip.isEmpty() || currentState.registerNumber == null) {
            return
        }
        connect(
            currentState.ip, currentState.firmWare,
            currentState.registerNumber.toString()
        )
    }

    data class ScreenState(
        val registerNumber: Int?,
        val ip: String,
        val firmWare: ConsoleVersion
    )

    companion object {
        private const val PIN_LENGTH = 8
    }

    private val _connectState: MutableStateFlow<ConnectState> =
        MutableStateFlow(ConnectState.Initial)
    val connectState = _connectState.asStateFlow()

    sealed class ConnectState {
        object Initial : ConnectState()
        object Connecting : ConnectState()
        object Connected : ConnectState()
        data class Failure(val cause: Exception) : ConnectState()
    }

    private fun connect(host: String, firmWare: ConsoleVersion, pin: String) =
        viewModelScope.launch(
            Dispatchers.IO
        ) {
            _connectState.value = ConnectState.Connecting
            val hostValid = host.isNotEmpty()
            val psnId = userRepository.getCurrentUser()?.userId.orEmpty()
            val psnOnlineId = if (firmWare == ConsoleVersion.PS4_LT_7) psnId else null
            val psnAccountIdByteArr =
                if (firmWare != ConsoleVersion.PS4_LT_7) {
                    BigInteger(psnId).toByteArray().reversedArray() // Little endian bytearr
                } else {
                    null
                }
            val psnIdValid = if (firmWare == ConsoleVersion.PS4_LT_7) {
                !psnOnlineId.isNullOrEmpty()
            } else {
                psnAccountIdByteArr != null
            }
            val pinValid = pin.length == PIN_LENGTH

            if (!hostValid || !psnIdValid || !pinValid) {
                handleRegisterFail(RegistException.InvalidInputException)
                return@launch
            }

            val target = when (firmWare) {
                ConsoleVersion.PS5 -> co.vulcanlabs.psremote.chiaki.Target.PS5_1
                ConsoleVersion.PS4_GE_8 -> co.vulcanlabs.psremote.chiaki.Target.PS4_10
                ConsoleVersion.PS4_GE_7 -> co.vulcanlabs.psremote.chiaki.Target.PS4_9
                ConsoleVersion.PS4_LT_7 -> co.vulcanlabs.psremote.chiaki.Target.PS4_8
            }
            val registInfo =
                RegistInfo(target, host, false, psnOnlineId, psnAccountIdByteArr, pin.toInt())
            Regist(
                registInfo,
                ChiakiLog(ChiakiLog.Level.DEBUG.value) { _, text -> text.showLog() },
                ::registEvent
            )
        }

    private fun registEvent(registEvent: RegistEvent) {
        when (registEvent) {
            is RegistEventFailed -> handleRegisterFail(RegistException.ConnectionFailException)
            is RegistEventCanceled -> handleRegisterFail(RegistException.ConnectionCancelException)
            is RegistEventSuccess -> handleRegisterSuccess(registEvent.host)
        }
    }

    private fun handleRegisterSuccess(host: RegistHost) = viewModelScope.launch {
        var result = deviceRepository.insertRegisterHost(RegisteredHost(host))
        if (registerSource == Source.Manually && result is Result.Success) {
            val insertedRegisteredHost =
                deviceRepository.getRegisterHostByMac(MacAddress(host.serverMac))
            if (insertedRegisteredHost is Result.Success) {
                result = deviceRepository.insertManualHost(
                    ManualHost(
                        host = screenStateFlow.value.ip,
                        registeredHost = insertedRegisteredHost.data.id
                    )
                )
            } else if (insertedRegisteredHost is Result.Failure) {
                result = Result.Failure(insertedRegisteredHost.exception)
            }
        }
        when (result) {
            is Result.Success -> _connectState.value = ConnectState.Connected
            is Result.Failure -> _connectState.value = ConnectState.Failure(result.exception)
        }
    }

    private fun handleRegisterFail(cause: Exception) {
        _connectState.value = ConnectState.Failure(cause)
    }

    fun setTutorialVideoId(@RawRes id: Int) {
        if (_tutorialVideoId.value != id) {
            _tutorialVideoId.value = id
        }
    }

    enum class Source {
        Manually, Select
    }
}