// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.touchcontrols

import android.content.Context
import android.graphics.Canvas
import android.graphics.drawable.Drawable
import android.util.AttributeSet
import android.view.MotionEvent
import android.view.View
import co.vulcanlabs.psremote.R
import kotlin.math.abs

class AnalogStickView @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : View(context, attrs, defStyleAttr) {
    val radius: Float
    private val handleRadius: Float
    private var currentState: DrawablesContainer.State = DrawablesContainer.State.IDLE

    var skipIncomingEvent = false

    private class DrawablesContainer(
        private val idleDrawable: Drawable?,
        private val pressedDrawable: Drawable?
    ) {

        enum class State {
            IDLE, PRESSED
        }

        fun setBounds(left: Int, top: Int, right: Int, bottom: Int) {
            idleDrawable?.setBounds(left, top, right, bottom)
            pressedDrawable?.setBounds(left, top, right, bottom)
        }

        fun draw(canvas: Canvas, state: State) {
            when (state) {
                State.IDLE -> idleDrawable?.draw(canvas)
                State.PRESSED -> pressedDrawable?.draw(canvas)
            }
        }
    }

    private val drawableBase: DrawablesContainer
    private val drawableHandle: DrawablesContainer

    var state = Vector(0f, 0f)
        private set(value) {
            field = value
            stateChangedCallback?.let { it(field) }
        }

    var stateChangedCallback: ((Vector) -> Unit)? = null

    var beginPressedCallback: (() -> Unit)? = null // For tracking purpose

    private val touchTracker = TouchTracker().also {
        it.positionChangedCallback = this::updateState
    }

    private var center: Vector? = null

    private val baseCenter: Vector by lazy {
        Vector((width / 2).toFloat(), (height / 2).toFloat())
    }

    /**
     * Same as state, but scaled to the circle
     */
    private var handlePosition: Vector = Vector(0f, 0f)

    init {
        context.theme.obtainStyledAttributes(attrs, R.styleable.AnalogStickView, 0, 0).apply {
            radius = getDimension(R.styleable.AnalogStickView_radius, 0f)
            handleRadius = getDimension(R.styleable.AnalogStickView_handleRadius, 0f)
            drawableBase = DrawablesContainer(
                getDrawable(R.styleable.AnalogStickView_drawableBaseIdle),
                getDrawable(R.styleable.AnalogStickView_drawableBasePressed)
            )
            drawableHandle = DrawablesContainer(
                getDrawable(R.styleable.AnalogStickView_drawableHandleIdle),
                getDrawable(R.styleable.AnalogStickView_drawableHandlePressed)
            )
            recycle()
        }
    }

    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)

        val center = center ?: baseCenter
        drawableBase.setBounds(
            (center.x - radius).toInt(),
            (center.y - radius).toInt(),
            (center.x + radius).toInt(),
            (center.y + radius).toInt()
        )
        drawableBase.draw(canvas, currentState)

        val handleX = center.x + handlePosition.x * radius
        val handleY = center.y + handlePosition.y * radius
        drawableHandle.setBounds(
            (handleX - handleRadius).toInt(),
            (handleY - handleRadius).toInt(),
            (handleX + handleRadius).toInt(),
            (handleY + handleRadius).toInt()
        )
        drawableHandle.draw(canvas, currentState)
    }

    private fun updateState(position: Vector?) {
        if (radius <= 0f) {
            return
        }

        if (position == null) {
            currentState = DrawablesContainer.State.IDLE
            center = null
            state = Vector(0f, 0f)
            handlePosition = Vector(0f, 0f)
            invalidate()
            return
        }
        currentState = DrawablesContainer.State.PRESSED

        val center: Vector = this.center ?: position
        this.center = center

        val dir = position - center
        val length = dir.length
        if (length > 0) {
            val strength = if (length > radius) 1.0f else length / radius
            val dirNormalized = dir / length
            handlePosition = dirNormalized * strength
            val dirBoxNormalized =
                if (abs(dirNormalized.x) > abs(dirNormalized.y)) {
                    dirNormalized / abs(dirNormalized.x)
                } else {
                    dirNormalized / abs(dirNormalized.y)
                }
            state = dirBoxNormalized * strength
        } else {
            handlePosition = Vector(0f, 0f)
            state = Vector(0f, 0f)
        }

        invalidate()
    }

    override fun onSizeChanged(w: Int, h: Int, oldw: Int, oldh: Int) {
        super.onSizeChanged(w, h, oldw, oldh)
        baseCenter.x = (w / 2).toFloat()
        baseCenter.y = (h / 2).toFloat()
        invalidate()
    }

    override fun onTouchEvent(event: MotionEvent): Boolean {
        if (event.actionMasked == MotionEvent.ACTION_DOWN || event.actionMasked == MotionEvent.ACTION_POINTER_DOWN) {
            beginPressedCallback?.invoke()
        }
        if (!skipIncomingEvent) {
            touchTracker.touchEvent(event)
        }
        return true
    }
}