// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.touchcontrols

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.ViewPropertyAnimator
import androidx.core.view.isInvisible
import androidx.core.view.isVisible
import androidx.fragment.app.Fragment
import androidx.fragment.app.activityViewModels
import androidx.lifecycle.LiveData
import androidx.lifecycle.Observer
import co.vulcanlabs.psremote.R
import co.vulcanlabs.psremote.chiaki.ControllerState
import co.vulcanlabs.psremote.databinding.FragmentControlsBinding
import co.vulcanlabs.psremote.ui.play.PlaySettingFragment
import co.vulcanlabs.psremote.ui.play.PlayViewModel
import dagger.hilt.android.AndroidEntryPoint
import io.reactivex.rxjava3.core.Observable
import io.reactivex.rxjava3.subjects.BehaviorSubject
import io.reactivex.rxjava3.subjects.Subject

abstract class TouchControlsFragment : Fragment() {
    protected var ownControllerState = ControllerState()
        set(value) {
            val diff = field != value
            field = value
            if (diff)
                ownControllerStateSubject.onNext(ownControllerState)
        }

    protected val ownControllerStateSubject: Subject<ControllerState> =
        BehaviorSubject.create<ControllerState>().also { it.onNext(ownControllerState) }

    // to delay attaching to the touchpadView until it's available
    protected val controllerStateProxy: Subject<Observable<ControllerState>> =
        BehaviorSubject.create<Observable<ControllerState>>()
            .also { it.onNext(ownControllerStateSubject) }
    val controllerState: Observable<ControllerState>
        get() =
            controllerStateProxy.flatMap { it }

    var onScreenControlsEnabled: LiveData<Boolean>? = null
}

@AndroidEntryPoint
class DefaultTouchControlsFragment : TouchControlsFragment() {
    private var _binding: FragmentControlsBinding? = null
    private val binding get() = _binding!!
    private var animator: ViewPropertyAnimator? = null

    private val playViewModel: PlayViewModel by activityViewModels()


    //    override fun onCreateView(
//        inflater: LayoutInflater,
//        container: ViewGroup?,
//        savedInstanceState: Bundle?
//    ): View =
//        FragmentControlsBinding.inflate(inflater, container, false).let {
//            _binding = it
//            controllerStateProxy.onNext(
//                combineLatest(
//                    ownControllerStateSubject,
//                    binding.touchpadView.controllerState
//                ) { a, b -> a or b }
//            )
//            it.root
//        }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View =
        FragmentControlsBinding.inflate(inflater, container, false).let {
            _binding = it
            controllerStateProxy.onNext(ownControllerStateSubject)
            it.root
        }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        binding.dpadView.stateChangeCallback = this::dpadStateChanged
        binding.faceButtonsView.buttonPressedCallback = { boolVal, faceType ->
            when (faceType) {
                FaceButtonView.ChildType.CROSS -> buttonStateChanged(ControllerState.BUTTON_CROSS).invoke(
                    boolVal
                )
                FaceButtonView.ChildType.PYRAMID -> buttonStateChanged(ControllerState.BUTTON_PYRAMID).invoke(
                    boolVal
                )
                FaceButtonView.ChildType.MOON -> buttonStateChanged(ControllerState.BUTTON_MOON).invoke(
                    boolVal
                )
                FaceButtonView.ChildType.BOX -> buttonStateChanged(ControllerState.BUTTON_BOX).invoke(
                    boolVal
                )
            }
        }

        binding.l1l2ButtonView.buttonPressedCallback = { pressed, faceType ->
            when (faceType) {
                L1L2ButtonView.ChildType.L1 -> buttonStateChanged(ControllerState.BUTTON_L1).invoke(
                    pressed
                )
                L1L2ButtonView.ChildType.L2 -> {
                    ownControllerState =
                        ownControllerState.copy().apply { l2State = if (pressed) 255U else 0U }
                }

            }
        }
        binding.r1r2ButtonView.buttonPressedCallback = { pressed, faceType ->
            when (faceType) {
                R1R2ButtonView.ChildType.R1 -> buttonStateChanged(ControllerState.BUTTON_R1).invoke(
                    pressed
                )
                R1R2ButtonView.ChildType.R2 -> {
                    ownControllerState =
                        ownControllerState.copy().apply { r2State = if (pressed) 255U else 0U }
                }

            }
        }
//        binding.l3ButtonView.buttonPressedCallback = buttonStateChanged(ControllerState.BUTTON_L3)
//        binding.r3ButtonView.buttonPressedCallback = buttonStateChanged(ControllerState.BUTTON_R3)
        binding.optionsButtonView.buttonPressedCallback =
            buttonStateChanged(ControllerState.BUTTON_OPTIONS)

        binding.shareButtonView.buttonPressedCallback =
            buttonStateChanged(ControllerState.BUTTON_SHARE)
        binding.psButtonView.buttonPressedCallback = buttonStateChanged(ControllerState.BUTTON_PS)

        binding.btnToggleBottomBar.setOnClickListener {
            toggleBottomHiddenBar()
        }
        binding.btnSetting.setOnClickListener {
            parentFragmentManager.run {
                if (findFragmentByTag(PlaySettingFragment.TAG)?.isRemoving != false) {
                    PlaySettingFragment.newInstance()
                        .show(this, PlaySettingFragment.TAG)
                }
            }
        }

        val quantizeStick = { f: Float ->
            (Short.MAX_VALUE * f).toInt().toShort()
        }

        binding.leftAnalogStickView.run {
            stateChangedCallback = {
                ownControllerState = ownControllerState.copy().apply {
                    leftX = quantizeStick(it.x)
                    leftY = quantizeStick(it.y)
                }
            }
        }

        binding.rightAnalogStickView.run {
            stateChangedCallback = {
                ownControllerState = ownControllerState.copy().apply {
                    rightX = quantizeStick(it.x)
                    rightY = quantizeStick(it.y)
                }
            }
        }

        onScreenControlsEnabled?.observe(viewLifecycleOwner, Observer {
            view.visibility = if (it) View.VISIBLE else View.GONE
        })

        binding.bottomHiddenBar.run {
            isInvisible =
                playViewModel.bottomBarState == PlayViewModel.BottomBarState.CLOSE
            post {
                translationY =
                    if (playViewModel.bottomBarState == PlayViewModel.BottomBarState.CLOSE) {
                        height.toFloat()
                    } else {
                        0f
                    }
            }
        }

        setImageForToggleButton()
    }

    private fun toggleBottomHiddenBar() {
        binding.bottomHiddenBar.post {
            playViewModel.bottomBarState =
                if (playViewModel.bottomBarState == PlayViewModel.BottomBarState.CLOSE) {
                    PlayViewModel.BottomBarState.OPEN
                } else {
                    PlayViewModel.BottomBarState.CLOSE
                }
            setImageForToggleButton()
            animator = if (playViewModel.bottomBarState == PlayViewModel.BottomBarState.CLOSE) {
                binding.bottomHiddenBar.animate()
                    .withEndAction {
                        binding.bottomHiddenBar.isInvisible = true
                    }
                    .translationY(binding.bottomHiddenBar.height.toFloat())
            } else {
                binding.bottomHiddenBar.animate()
                    .withStartAction {
                        binding.bottomHiddenBar.isVisible = true
                    }
                    .translationY(0f)
            }.also {
                it.start()
            }
        }
    }

    private fun setImageForToggleButton() {
        binding.btnToggleBottomBar.setImageResource(
            if (playViewModel.bottomBarState == PlayViewModel.BottomBarState.CLOSE) {
                R.drawable.ic_more_with_circle
            } else {
                R.drawable.ic_close
            }
        )
    }

    private fun dpadStateChanged(direction: DPadView.Direction?) {
        ownControllerState = ownControllerState.copy().apply {
            buttons = ((buttons
                    and ControllerState.BUTTON_DPAD_LEFT.inv()
                    and ControllerState.BUTTON_DPAD_RIGHT.inv()
                    and ControllerState.BUTTON_DPAD_UP.inv()
                    and ControllerState.BUTTON_DPAD_DOWN.inv())
                    or when (direction) {
                DPadView.Direction.UP -> ControllerState.BUTTON_DPAD_UP
                DPadView.Direction.DOWN -> ControllerState.BUTTON_DPAD_DOWN
                DPadView.Direction.LEFT -> ControllerState.BUTTON_DPAD_LEFT
                DPadView.Direction.RIGHT -> ControllerState.BUTTON_DPAD_RIGHT
                DPadView.Direction.LEFT_UP -> ControllerState.BUTTON_DPAD_LEFT or ControllerState.BUTTON_DPAD_UP
                DPadView.Direction.LEFT_DOWN -> ControllerState.BUTTON_DPAD_LEFT or ControllerState.BUTTON_DPAD_DOWN
                DPadView.Direction.RIGHT_UP -> ControllerState.BUTTON_DPAD_RIGHT or ControllerState.BUTTON_DPAD_UP
                DPadView.Direction.RIGHT_DOWN -> ControllerState.BUTTON_DPAD_RIGHT or ControllerState.BUTTON_DPAD_DOWN
                null -> 0U
            })
        }
    }

    private fun buttonStateChanged(buttonMask: UInt) = { pressed: Boolean ->
        ownControllerState = ownControllerState.copy().apply {
            buttons =
                if (pressed) buttons or buttonMask else buttons and buttonMask.inv()

        }
    }
}