// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.utils

import android.app.Activity
import android.app.Application
import android.content.ActivityNotFoundException
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.content.pm.PackageManager
import android.content.res.Resources
import android.net.ConnectivityManager
import android.net.NetworkCapabilities
import android.net.Uri
import android.os.Build
import android.util.Log
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.InputMethodManager
import android.widget.PopupWindow
import android.widget.Toast
import android.widget.VideoView
import androidx.annotation.IdRes
import androidx.annotation.RawRes
import androidx.core.content.ContextCompat
import androidx.core.content.getSystemService
import androidx.fragment.app.DialogFragment
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentActivity
import androidx.lifecycle.LifecycleCoroutineScope
import androidx.navigation.fragment.findNavController
import co.vulcanlabs.psremote.BuildConfig
import co.vulcanlabs.psremote.MyApplication
import co.vulcanlabs.psremote.R
import co.vulcanlabs.psremote.common.ImprovedDateTypeAdapter
import co.vulcanlabs.psremote.common.annotation.GsonDeserializationIgnore
import co.vulcanlabs.psremote.common.annotation.GsonIgnore
import co.vulcanlabs.psremote.ui.alertdialog.LoadingFragment
import com.google.gson.ExclusionStrategy
import com.google.gson.FieldAttributes
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.flow.collectLatest
import okhttp3.RequestBody
import okio.Buffer
import java.io.IOException
import java.util.*


fun Context.checkAppPermission(list: List<String>): Boolean {
    return list.map { checkCallingOrSelfPermission(it) }
        .all { it == PackageManager.PERMISSION_GRANTED }
}

inline fun <reified T> Context.startActivityView(block: (Intent.() -> Unit)) {
    try {
        startActivity(Intent(this, T::class.java).apply { block.invoke(this) })
    } catch (e: Exception) {
    }
}

fun Context.dpToPx(dp: Int) = dp * resources.displayMetrics.density

fun Context.spToPx(sp: Int): Int {
    return TypedValue.applyDimension(
        TypedValue.COMPLEX_UNIT_SP,
        sp.toFloat(),
        resources.displayMetrics
    ).toInt()
}

fun String.hexToByteArray(): ByteArray? = ByteArray(this.length / 2) {
    this.substring(it * 2, it * 2 + 2).toIntOrNull(16)?.toByte() ?: return null
}

fun RequestBody?.bodyToString(): String? {
    return try {
        val buffer = Buffer()
        if (this != null) this.writeTo(buffer) else return ""
        buffer.readUtf8()
    } catch (e: IOException) {
        "did not work"
    }
}

fun Int.toBytesArray(): ByteArray {
    return byteArrayOf(
        (this shr 24 and 0xff).toByte(),
        (this shr 16 and 0xff).toByte(),
        (this shr 8 and 0xff).toByte(),
        (this shr 0 and 0xff).toByte()
    )
}

fun <T> Flow<T>.collectWhenStarted(
    lifeCycleScope: LifecycleCoroutineScope,
    action: suspend (value: T) -> Unit
) =
    lifeCycleScope.launchWhenStarted {
        collect(action)
    }

fun <T> Flow<T>.collectLatestWhenStarted(
    lifeCycleScope: LifecycleCoroutineScope,
    action: suspend (value: T) -> Unit
) =
    lifeCycleScope.launchWhenStarted {
        collectLatest(action)
    }

fun <T> Flow<T>.collectLatestWhenResumed(
    lifeCycleScope: LifecycleCoroutineScope,
    action: suspend (value: T) -> Unit
) =
    lifeCycleScope.launchWhenResumed {
        collectLatest(action)
    }

fun FragmentActivity.showDialog(dialog: DialogFragment, tag: String) {
    if (supportFragmentManager.findFragmentByTag(tag)?.isRemoving != false) {
        dialog.showNow(supportFragmentManager, tag)
    }
}

fun Fragment.showDialog(dialog: DialogFragment, tag: String, showAsChild: Boolean = true) {
    if (showAsChild) {
        if (childFragmentManager.findFragmentByTag(tag)?.isRemoving != false) {
            dialog.showNow(childFragmentManager, tag)
        }
    } else {
        activity?.showDialog(dialog, tag)
    }
}


/**
 * Fix ANR showing PopupWindow when keyboard was opened before
 */
fun PopupWindow.showAsDropDownSafe(anchor: View, activity: Activity) {
    activity.hideKeyboard()
    anchor.post {
        showAsDropDown(anchor)
    }
}

fun Activity.hideKeyboard() {
    (currentFocus ?: View(this)).hideKeyboard()
}

fun View.hideKeyboard() {
    post {
        val imm = context.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        imm.hideSoftInputFromWindow(windowToken, 0)
    }
}

fun FragmentActivity.showAppLoading() {
    showDialog(LoadingFragment.newInstance(), LoadingFragment.TAG)
}

fun FragmentActivity.hideAppLoading() {
    (supportFragmentManager.findFragmentByTag(LoadingFragment.TAG)
        .takeIf { it is DialogFragment && !it.isRemoving } as? DialogFragment)?.let {
        it.dismissAllowingStateLoss()
    }
}

fun Fragment.showAppLoading() {
    activity?.showAppLoading()
}

fun Fragment.hideAppLoading() {
    activity?.hideAppLoading()
}

fun VideoView.setUp(@RawRes rawRes: Int, container: ViewGroup? = null) {
    val target = container ?: this
    val uri = Uri.parse("android.resource://${context.packageName}/$rawRes")
    try {
        setVideoURI(uri)
        start()
        setOnPreparedListener { mediaPlayer ->
            mediaPlayer.isLooping = true
            mediaPlayer.setVolume(0f, 0f)
            val height =
                target.width * mediaPlayer.videoHeight / mediaPlayer.videoWidth
            target.layoutParams = target.layoutParams.apply {
                this.height = height
            }

        }
    } catch (e: IllegalStateException) {
    }
}

fun Context.getNavigationBarHeight(): Int {
    val resourceId: Int = resources.getIdentifier("navigation_bar_height", "dimen", "android")
    return if (resourceId > 0) {
        resources.getDimensionPixelSize(resourceId)
    } else 0
}

fun View.setClickableAnimationBorderless() {
    if (Build.VERSION.SDK_INT >= 23) {
        val outValue = TypedValue()
        context.theme.resolveAttribute(
            R.attr.selectableItemBackgroundBorderless, outValue, true
        )
        foreground = ContextCompat.getDrawable(context, outValue.resourceId)
    }
}

fun Fragment.navigate(@IdRes action: Int) {
    try {
        findNavController().navigate(action)
    } catch (e: Exception) {
    }
}

fun String.showLog(tag: String = "CUSTOM_LOG") {
    if (BuildConfig.DEBUG) {
        Log.d(tag, this)
    }
}

fun Context.openUrlBrowser(url: String, urlTitle: String) {
    startActivity(
        Intent.createChooser(
            Intent(Intent.ACTION_VIEW).setData(Uri.parse(url)),
            urlTitle
        ).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
    )
}

fun String.showToast(context: Context, isLong: Boolean = false): String {
    val length = if (isLong) Toast.LENGTH_LONG else Toast.LENGTH_SHORT
    Toast.makeText(context, this, length).show()
    return this
}

fun String.showToast(isLong: Boolean) {
    val length = if (isLong) Toast.LENGTH_LONG else Toast.LENGTH_SHORT
    Toast.makeText(MyApplication.instance, this, length).show()
}

fun Context.showToast(msg: String): String {
    Toast.makeText(
        this,
        msg,
        Toast.LENGTH_SHORT
    ).show()
    return msg
}

fun <T> getViewBinding(inflater: LayoutInflater, clazz: Class<T>) =
    clazz.getMethod("inflate", LayoutInflater::class.java)
        .invoke(null, inflater)
        .convert<T>()

@Suppress("UNCHECKED_CAST")
fun <T> Any.convert() = this as T

fun Context.showAppInPlayStore() {
    try {
        startActivity(
            Intent(
                Intent.ACTION_VIEW,
                Uri.parse("market://details?id=$packageName")
            ).setFlags(
                Intent.FLAG_ACTIVITY_NEW_TASK
            )
        )
    } catch (anfe: ActivityNotFoundException) {
        startActivity(
            Intent(
                Intent.ACTION_VIEW,
                Uri.parse("https://play.google.com/store/apps/details?id=$packageName")
            ).setFlags(
                Intent.FLAG_ACTIVITY_NEW_TASK
            )
        )
    }
}

fun Context.showOtherApps() {
    try {
        val intent = Intent(
            Intent.ACTION_VIEW,
            Uri.parse("https://play.google.com/store/apps/dev?id=9175334228948628001")
        ).setFlags(
            Intent.FLAG_ACTIVITY_NEW_TASK
        )
        startActivity(intent)
    } catch (e: Exception) {
    }
}

fun Context.getSharePref(): SharedPreferences {
    return getSharedPreferences(packageName, Context.MODE_PRIVATE)
}

fun getGsonInstance(): Gson {
    return GsonBuilder().serializeNulls()
        .registerTypeAdapter(Date::class.java, ImprovedDateTypeAdapter())
        .addSerializationExclusionStrategy(object : ExclusionStrategy {
            override fun shouldSkipField(f: FieldAttributes): Boolean {
                return (f.getAnnotation(GsonDeserializationIgnore::class.java) != null) ||
                        f.getAnnotation(GsonIgnore::class.java) != null
            }

            override fun shouldSkipClass(clazz: Class<*>): Boolean {
                return clazz.getAnnotation(GsonDeserializationIgnore::class.java) != null ||
                        clazz.getAnnotation(GsonIgnore::class.java) != null
            }
        })
        .addDeserializationExclusionStrategy(object : ExclusionStrategy {
            override fun shouldSkipField(f: FieldAttributes): Boolean {
                return f.getAnnotation(GsonDeserializationIgnore::class.java) != null ||
                        f.getAnnotation(GsonIgnore::class.java) != null
            }

            override fun shouldSkipClass(clazz: Class<*>): Boolean {
                return clazz.getAnnotation(GsonDeserializationIgnore::class.java) != null ||
                        clazz.getAnnotation(GsonIgnore::class.java) != null
            }
        })
        .create()
}

fun Any.toJson() =
    try {
        getGsonInstance().toJson(this)
    } catch (e: Exception) {
        ""
    }

inline fun <reified T> String.jsonToObj() = getGsonInstance()
    .fromJson(this, T::class.java)

@Suppress("DEPRECATION")
fun Application.isNetworkConnected(): Boolean {
    val cm = getSystemService(Context.CONNECTIVITY_SERVICE).convert<ConnectivityManager>()
    return if (Build.VERSION.SDK_INT < 23) {
        cm.activeNetworkInfo
            ?.let { ni ->
                ni.isConnected && (ni.type == ConnectivityManager.TYPE_WIFI || ni.type == ConnectivityManager.TYPE_MOBILE)
            } ?: false
    } else {
        cm.activeNetwork
            ?.let { cm.getNetworkCapabilities(it) }
            ?.let { nc ->
                nc.hasTransport(NetworkCapabilities.TRANSPORT_CELLULAR) ||
                        nc.hasTransport(NetworkCapabilities.TRANSPORT_WIFI)
            }
            ?: false
    }
}

fun Int.dpToPx(): Int {
    return (this * Resources.getSystem().displayMetrics.density).toInt()
}
